@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dvdnav.h,v 1.31 2003/06/09 15:17:44 mroi Exp $
+ * $Id: dvdnav.h 955 2007-08-19 21:25:04Z rathann $
  *
  */
 
@@ -29,20 +29,18 @@
 #ifndef DVDNAV_H_INCLUDED
 #define DVDNAV_H_INCLUDED
 
+#define MP_DVDNAV 1
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifdef DVDNAV_COMPILE
-#  include "dvdnav_events.h"
-#  include "dvd_types.h"
-#  include "dvd_reader.h"
-#  include "ifo_types.h" /* For vm_cmd_t */
-#else
-#  include <dvdnav/dvdnav_events.h>
+#ifndef DVDNAV_COMPILE
 #  include <dvdnav/dvd_types.h>
-#  include <dvdnav/dvd_reader.h>
-#  include <dvdnav/ifo_types.h> /* For vm_cmd_t */
+#  include <dvdread/dvd_reader.h>
+#  include <dvdread/nav_types.h>
+#  include <dvdread/ifo_types.h> /* For vm_cmd_t */
+#  include <dvdnav/dvdnav_events.h>
 #endif
 
 
@@ -70,6 +68,11 @@ typedef int32_t dvdnav_status_t;
 #define DVDNAV_STATUS_ERR 0
 #define DVDNAV_STATUS_OK  1
 
+#define DVDNAV_FORMAT_AC3 0
+#define DVDNAV_FORMAT_MPEGAUDIO 3
+#define DVDNAV_FORMAT_LPCM 4
+#define DVDNAV_FORMAT_DTS 5
+#define DVDNAV_FORMAT_SDDS 6
 
 /*********************************************************************
  * initialisation & housekeeping functions                           *
@@ -353,6 +356,12 @@ dvdnav_status_t dvdnav_part_search(dvdnav_t *self, int32_t part);
 dvdnav_status_t dvdnav_sector_search(dvdnav_t *self, 
 				     uint64_t offset, int32_t origin);
 
+/*
+ returns the current stream time in PTS ticks as reported by the IFO structures
+ divide it by 90000 to get the current play time in seconds
+ */
+int64_t dvdnav_get_current_time(dvdnav_t *self);
+
 /*
  * Stop playing the current position and start playback of the title
  * from the specified timecode.
@@ -547,6 +556,18 @@ uint8_t dvdnav_get_video_scale_permission(dvdnav_t *self);
  */
 uint16_t dvdnav_audio_stream_to_lang(dvdnav_t *self, uint8_t stream);
 
+/*
+ * Returns the format of *logical* audio stream 'stream'
+ * (returns 0xffff if no such stream).
+ */
+uint16_t dvdnav_audio_stream_format(dvdnav_t *self, uint8_t stream);
+
+/*
+ * Returns number of channelsn in *logical* audio stream 'stream'
+ * (returns 0xffff if no such stream).
+ */
+uint16_t dvdnav_audio_stream_channels(dvdnav_t *self, uint8_t stream);
+
 /*
  * Converts a *logical* subpicture stream id into country code 
  * (returns 0xffff if no such stream).
@@ -558,11 +579,23 @@ uint16_t dvdnav_spu_stream_to_lang(dvdnav_t *self, uint8_t stream);
  */
 int8_t dvdnav_get_audio_logical_stream(dvdnav_t *self, uint8_t audio_num);
 
+#define HAVE_GET_AUDIO_ATTR
+/*
+ * Get audio attr
+ */
+dvdnav_status_t dvdnav_get_audio_attr(dvdnav_t *self, uint8_t audio_mum, audio_attr_t *audio_attr);
+
 /*
  * Converts a *physical* (MPEG) subpicture stream id into a logical stream number.
  */
 int8_t dvdnav_get_spu_logical_stream(dvdnav_t *self, uint8_t subp_num);
 
+#define HAVE_GET_SPU_ATTR
+/*
+ * Get spu attr
+ */
+dvdnav_status_t dvdnav_get_spu_attr(dvdnav_t *self, uint8_t audio_mum, subp_attr_t *subp_attr);
+
 /*
  * Get active audio stream.
  */
@@ -573,6 +606,13 @@ int8_t dvdnav_get_active_audio_stream(dvdnav_t *self);
  */
 int8_t dvdnav_get_active_spu_stream(dvdnav_t *self);
 
+/* 
+ * Get the set of user operations that are currently prohibited.
+ * There are potentially new restrictions right after 
+ * DVDNAV_CHANNEL_HOP and DVDNAV_NAV_PACKET.
+ */
+user_ops_t dvdnav_get_restrictions(dvdnav_t *self);
+
 
 /*********************************************************************
  * multiple angles                                                   *
