@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2001 Rich Wareham <richwareham@users.sourceforge.net>
+ * Copyright (C) 2001-2004 Rich Wareham <richwareham@users.sourceforge.net>
  * 
  * This file is part of libdvdnav, a DVD navigation library.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dvdnav_internal.h,v 1.12 2004/02/13 19:16:17 mroi Exp $
+ * $Id: dvdnav_internal.h 891 2007-04-21 11:56:41Z nicodvb $
  *
  */
 
@@ -28,12 +28,6 @@
 #include "config.h"
 #endif
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <limits.h>
-#include <string.h>
-
 #ifdef WIN32
 
 /* pthread_mutex_* wrapper for win32 */
@@ -47,7 +41,7 @@ typedef CRITICAL_SECTION pthread_mutex_t;
 
 /* replacement gettimeofday implementation */
 #include <sys/timeb.h>
-static inline int gettimeofday( struct timeval *tv, void *tz )
+static inline int _private_gettimeofday( struct timeval *tv, void *tz )
 {
   struct timeb t;
   ftime( &t );
@@ -55,6 +49,7 @@ static inline int gettimeofday( struct timeval *tv, void *tz )
   tv->tv_usec = t.millitm * 1000;
   return 0;
 }
+#define gettimeofday(TV, TZ) _private_gettimeofday((TV), (TZ))
 #include <io.h> /* read() */
 #define lseek64 _lseeki64
 
@@ -64,14 +59,6 @@ static inline int gettimeofday( struct timeval *tv, void *tz )
 
 #endif /* WIN32 */
 
-/* Uncomment for VM command tracing */
-/* #define TRACE */
-
-#include "decoder.h"
-#include "dvdnav.h"
-#include "vm.h"
-#include "vmcmd.h"
-
 /* where should libdvdnav write its messages (stdout/stderr) */
 #define MSG_OUT stdout
 
@@ -170,6 +157,7 @@ struct dvdnav_s {
   int started;                    /* vm_start has been called? */
   int use_read_ahead;             /* 1 - use read-ahead cache, 0 - don't */
   int pgc_based;                  /* positioning works PGC based instead of PG based */
+  int cur_cell_time;              /* time expired since the beginning of the current cell, read from the dsi */
   
   /* VM */
   vm_t *vm;
@@ -182,17 +170,26 @@ struct dvdnav_s {
   char err_str[MAX_ERR_LEN];
 };
 
+/** HELPER FUNCTIONS **/
+
+/* converts a dvd_time_t to PTS ticks */
+int64_t dvdnav_convert_time(dvd_time_t *time);
+
 /** USEFUL MACROS **/
 
 #ifdef __GNUC__
-#define printerrf(format, args...) snprintf(this->err_str, MAX_ERR_LEN, format, ## args);
+#define printerrf(format, args...) \
+	do { if (this) snprintf(this->err_str, MAX_ERR_LEN, format, ## args); } while (0)
 #else
 #ifdef _MSC_VER
-#define printerrf(str) snprintf(this->err_str, MAX_ERR_LEN, str);
+#define printerrf(str) \
+	do { if (this) snprintf(this->err_str, MAX_ERR_LEN, str); } while (0)
 #else
-#define printerrf(...) snprintf(this->err_str, MAX_ERR_LEN, __VA_ARGS__);
+#define printerrf(...) \
+	do { if (this) snprintf(this->err_str, MAX_ERR_LEN, __VA_ARGS__); } while (0)
 #endif /* WIN32 */
 #endif
-#define printerr(str) strncpy(this->err_str, str, MAX_ERR_LEN);
+#define printerr(str) \
+	do { if (this) strncpy(this->err_str, str, MAX_ERR_LEN - 1); } while (0)
 
 #endif /* DVDNAV_INTERNAL_H_INCLUDED */
