@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2001, 2002, 2003 Billy Biggs <vektor@dumbterm.net>,
- *                                Hkan Hjort <d95hjort@dtek.chalmers.se>,
- *                                Bjrn Englund <d4bjorn@dtek.chalmers.se>
+ * Copyright (C) 2001-2004 Billy Biggs <vektor@dumbterm.net>,
+ *                         Hkan Hjort <d95hjort@dtek.chalmers.se>,
+ *                         Bjrn Englund <d4bjorn@dtek.chalmers.se>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 #ifdef WIN32
 /* replacement gettimeofday implementation */
 #include <sys/timeb.h>
-static inline int gettimeofday( struct timeval *tv, void *tz )
+static inline int _private_gettimeofday( struct timeval *tv, void *tz )
 {
   struct timeb t;
   ftime( &t );
@@ -44,6 +44,7 @@ static inline int gettimeofday( struct timeval *tv, void *tz )
   tv->tv_usec = t.millitm * 1000;
   return 0;
 }
+#define gettimeofday(TV, TZ) _private_gettimeofday((TV), (TZ))
 #include <io.h> /* read() */
 #define lseek64 _lseeki64
 #endif
@@ -331,6 +332,8 @@ dvd_reader_t *DVDOpen( const char *ppath )
       return 0;
 
 	path = strdup(ppath);
+    if( path == NULL )
+      return 0;
 	
     /* Try to open libdvdcss or fall back to standard functions */
     have_css = dvdinput_setup();
@@ -585,10 +588,12 @@ static int findDirFile( const char *path, const char *file, char *filename )
             sprintf( filename, "%s%s%s", path,
                      ( ( path[ strlen( path ) - 1 ] == '/' ) ? "" : "/" ),
                      ent->d_name );
+            closedir( dir );
             return 0;
         }
     }
 
+    closedir( dir );
     return -1;
 }
 
