@@ -13,7 +13,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: ifo_print.c,v 1.1 2004/01/11 21:43:13 mroi Exp $
+ * $Id: ifo_print.c 952 2007-08-02 21:58:59Z nicodvb $
  *
  */
 
@@ -71,7 +71,10 @@ static void ifo_print_cmd(int row, vm_cmd_t *command) {
   for(i=0;i<8;i++)
     printf("%02x ", command->bytes[i]);
   printf("| ");
+#if 0
+  //disabled call of dvdnav function
   vm_print_mnemonic(command);
+#endif
   printf("\n");
 }
 
@@ -704,6 +707,10 @@ static void ifo_print_CELL_POSITION(cell_position_t *cell_position, int nr) {
 void ifo_print_PGC(pgc_t *pgc) {
   int i;
   
+  if (!pgc) {
+    printf("None\n");
+    return;
+  }
   printf("Number of Programs: %i\n", pgc->nr_of_programs);
   printf("Number of Cells: %i\n", pgc->nr_of_cells);
   /* Check that time is 0:0:0:0 also if nr_of_programs==0 */
@@ -723,8 +730,12 @@ void ifo_print_PGC(pgc_t *pgc) {
   
   for(i = 0; i < 32; i++) {
     if(pgc->subp_control[i] & 0x80000000) { /* The 'is present' bit */
-      printf("Subpicture stream %2i control: %08x\n", 
-	     i, pgc->subp_control[i]);
+      printf("Subpicture stream %2i control: %08x: 4:3=%d, Wide=%d, Letterbox=%d, Pan-Scan=%d\n", 
+	     i, pgc->subp_control[i],
+	     (pgc->subp_control[i] >>24) & 0x1f,
+	     (pgc->subp_control[i] >>16) & 0x1f,
+	     (pgc->subp_control[i] >>8) & 0x1f,
+	     (pgc->subp_control[i] ) & 0x1f);
     }
   }
   
@@ -1163,7 +1174,13 @@ void ifo_print(dvd_reader_t *dvd, int title) {
 }
 
 /*
- * $Log: ifo_print.c,v $
+ * $Log$
+ * Revision 1.3  2004/10/22 11:36:19  jcdutton
+ * Stop seg fault when using ifo_dump.
+ *
+ * Revision 1.2  2004/09/27 12:24:01  jcdutton
+ * Add extra info to printout when using ifo_dump.
+ *
  * Revision 1.1  2004/01/11 21:43:13  mroi
  * big build system changes
  *  * cleaned up all Makefiles and added a Makefile.common
