@@ -19,7 +19,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: decoder.c,v 1.2 2004/03/03 16:50:42 mroi Exp $
+ * $Id: decoder.c 900 2007-04-24 23:06:22Z nicodvb $
  *
  */
 
@@ -30,9 +30,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <inttypes.h>
+#include <limits.h>
 #include <string.h>  /* For memset */
+#include <sys/time.h>
+#include "nav_types.h"
 #include "ifo_types.h" /* vm_cmd_t */
 
+#include "dvd_types.h"
+#include "remap.h"
+#include "decoder.h"
+#include "vm.h"
+#include "vmcmd.h"
+#include "dvdnav.h"
 #include "dvdnav_internal.h"
 
 uint32_t vm_getbits(command_t *command, int32_t start, int32_t count) {
@@ -572,7 +581,7 @@ static int32_t eval_command(uint8_t *bytes, registers_t* registers, link_t *retu
 
   if(command.instruction & ~ command.examined) {
     fprintf(MSG_OUT, "libdvdnav: decoder.c: [WARNING, unknown bits:");
-    fprintf(MSG_OUT, " %08llx", (command.instruction & ~ command.examined) );
+    fprintf(MSG_OUT, " %08"PRIx64, (command.instruction & ~ command.examined) );
     fprintf(MSG_OUT, "]\n");
   }
 
