@@ -20,7 +20,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: vm.c,v 1.6 2004/05/21 19:32:25 mroi Exp $
+ * $Id: vm.c 900 2007-04-24 23:06:22Z nicodvb $
  *
  */
 
@@ -33,14 +33,23 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <inttypes.h>
+#include <limits.h>
 #include <assert.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <fcntl.h>
 
+#include "nav_types.h"
 #include "ifo_types.h"
 #include "ifo_read.h"
+#include "dvd_types.h"
 
+#include "decoder.h"
+#include "remap.h"
+#include "vm.h"
+#include "vmcmd.h"
+#include "dvdnav.h"
 #include "dvdnav_internal.h"
 
 #ifdef _MSC_VER
@@ -184,9 +193,9 @@ static void dvd_read_name(char *name, const char *device) {
   }
 }
 
-static void ifoOpenNewVTSI(vm_t *vm, dvd_reader_t *dvd, int vtsN) {
+static int ifoOpenNewVTSI(vm_t *vm, dvd_reader_t *dvd, int vtsN) {
   if((vm->state).vtsN == vtsN) {
-    return; /*  We alread have it */
+    return 1; /*  We alread have it */
   }
   
   if(vm->vtsi != NULL)
@@ -194,30 +203,32 @@ static void ifoOpenNewVTSI(vm_t *vm, dvd_reader_t *dvd, int vtsN) {
   
   vm->vtsi = ifoOpenVTSI(dvd, vtsN);
   if(vm->vtsi == NULL) {
-    fprintf(MSG_OUT, "libdvdnav: ifoOpenVTSI failed - CRASHING!!!\n");
-    assert(0);
+    fprintf(MSG_OUT, "libdvdnav: ifoOpenVTSI failed\n");
+    return 0;
   }
   if(!ifoRead_VTS_PTT_SRPT(vm->vtsi)) {
-    fprintf(MSG_OUT, "libdvdnav: ifoRead_VTS_PTT_SRPT failed - CRASHING!!!\n");
-    assert(0);
+    fprintf(MSG_OUT, "libdvdnav: ifoRead_VTS_PTT_SRPT failed\n");
+    return 0;
   }
   if(!ifoRead_PGCIT(vm->vtsi)) {
-    fprintf(MSG_OUT, "libdvdnav: ifoRead_PGCIT failed - CRASHING!!!\n");
-    assert(0);
+    fprintf(MSG_OUT, "libdvdnav: ifoRead_PGCIT failed\n");
+    return 0;
   }
   if(!ifoRead_PGCI_UT(vm->vtsi)) {
-    fprintf(MSG_OUT, "libdvdnav: ifoRead_PGCI_UT failed - CRASHING!!!\n");
-    assert(0);
+    fprintf(MSG_OUT, "libdvdnav: ifoRead_PGCI_UT failed\n");
+    return 0;
   }
   if(!ifoRead_VOBU_ADMAP(vm->vtsi)) {
-    fprintf(MSG_OUT, "libdvdnav: ifoRead_VOBU_ADMAP vtsi failed - CRASHING\n");
-    assert(0);
+    fprintf(MSG_OUT, "libdvdnav: ifoRead_VOBU_ADMAP vtsi failed\n");
+    return 0;
   }
   if(!ifoRead_TITLE_VOBU_ADMAP(vm->vtsi)) {
-    fprintf(MSG_OUT, "libdvdnav: ifoRead_TITLE_VOBU_ADMAP vtsi failed - CRASHING\n");
-    assert(0);
+    fprintf(MSG_OUT, "libdvdnav: ifoRead_TITLE_VOBU_ADMAP vtsi failed\n");
+    return 0;
   }
   (vm->state).vtsN = vtsN;
+  
+  return 1;
 }
 
 
@@ -253,10 +264,11 @@ dvd_reader_t *vm_get_dvd_reader(vm_t *vm) {
 
 /* Basic Handling */
 
-void vm_start(vm_t *vm) {
+int vm_start(vm_t *vm) {
   /* Set pgc to FP (First Play) pgc */
   set_FP_PGC(vm);
   process_command(vm, play_PGC(vm));
+  return !vm->stopped;
 }
 
 void vm_stop(vm_t *vm) {
@@ -316,14 +328,14 @@ int vm_reset(vm_t *vm, const char *dvdroot) {
   if (!vm->dvd) {
     vm->dvd = DVDOpen(dvdroot);
     if(!vm->dvd) {
-      fprintf(MSG_OUT, "libdvdnav: vm: faild to open/read the DVD\n");
+      fprintf(MSG_OUT, "libdvdnav: vm: failed to open/read the DVD\n");
       return 0;
     }
     dvd_read_name(vm->dvd_name, dvdroot);
     vm->map  = remap_loadmap(vm->dvd_name);
     vm->vmgi = ifoOpenVMGI(vm->dvd);
     if(!vm->vmgi) {
-      fprintf(MSG_OUT, "libdvdnav: vm: faild to read VIDEO_TS.IFO\n");
+      fprintf(MSG_OUT, "libdvdnav: vm: failed to read VIDEO_TS.IFO\n");
       return 0;
     }
     if(!ifoRead_FP_PGC(vm->vmgi)) {
@@ -382,7 +394,8 @@ vm_t *vm_new_copy(vm_t *source) {
   vtsN = (target->state).vtsN;
   if (vtsN > 0) {
     (target->state).vtsN = 0;
-    ifoOpenNewVTSI(target, target->dvd, vtsN);
+    if (!ifoOpenNewVTSI(target, target->dvd, vtsN))
+      assert(0);
   
     /* restore pgc pointer into the new vtsi */
     if (!set_PGCN(target, pgcN))
@@ -712,8 +725,8 @@ int vm_get_subp_stream(vm_t *vm, int subpN, int mode) {
     }
   }
   
- if((vm->state).domain != VTS_DOMAIN && streamN == -1)
-   streamN = 0;
+  if((vm->state).domain != VTS_DOMAIN && streamN == -1)
+    streamN = 0;
 
   /* FIXME: Should also check in vtsi/vmgi status what kind of stream it is. */
   return streamN;
@@ -1280,9 +1293,10 @@ static int process_command(vm_t *vm, link_t link_values) {
       if(link_values.data1 != 0)
 	(vm->state).HL_BTNN_REG = link_values.data1 << 10;
       assert((vm->state).pgc->next_pgc_nr != 0);
-      if(!set_PGCN(vm, (vm->state).pgc->next_pgc_nr))
-	assert(0);
-      link_values = play_PGC(vm);
+      if(set_PGCN(vm, (vm->state).pgc->next_pgc_nr))
+	link_values = play_PGC(vm);
+      else
+	link_values.command = Exit;
       break;
     case LinkPrevPGC:
       /* Link to Previous Program Chain */
@@ -1290,9 +1304,10 @@ static int process_command(vm_t *vm, link_t link_values) {
       if(link_values.data1 != 0)
 	(vm->state).HL_BTNN_REG = link_values.data1 << 10;
       assert((vm->state).pgc->prev_pgc_nr != 0);
-      if(!set_PGCN(vm, (vm->state).pgc->prev_pgc_nr))
-	assert(0);
-      link_values = play_PGC(vm);
+      if(set_PGCN(vm, (vm->state).pgc->prev_pgc_nr))
+	link_values = play_PGC(vm);
+      else
+	link_values.command = Exit;
       break;
     case LinkGoUpPGC:
       /* Link to GoUp Program Chain */
@@ -1300,9 +1315,10 @@ static int process_command(vm_t *vm, link_t link_values) {
       if(link_values.data1 != 0)
 	(vm->state).HL_BTNN_REG = link_values.data1 << 10;
       assert((vm->state).pgc->goup_pgc_nr != 0);
-      if(!set_PGCN(vm, (vm->state).pgc->goup_pgc_nr))
-	assert(0);
-      link_values = play_PGC(vm);
+      if(set_PGCN(vm, (vm->state).pgc->goup_pgc_nr))
+	link_values = play_PGC(vm);
+      else
+	link_values.command = Exit;
       break;
     case LinkTailPGC:
       /* Link to Tail of Program Chain */
@@ -1325,7 +1341,8 @@ static int process_command(vm_t *vm, link_t link_values) {
 	}
 	
 	(vm->state).domain = VTS_DOMAIN;
-	ifoOpenNewVTSI(vm, vm->dvd, (vm->state).rsm_vtsN);
+	if (!ifoOpenNewVTSI(vm, vm->dvd, (vm->state).rsm_vtsN))
+	  assert(0);
 	set_PGCN(vm, (vm->state).rsm_pgcN);
 	
 	/* These should never be set in SystemSpace and/or MenuSpace */ 
@@ -1343,10 +1360,11 @@ static int process_command(vm_t *vm, link_t link_values) {
 	  (vm->state).pgN = 1;
 	  link_values = play_PG(vm);
 	} else { 
-	  /* (vm->state).pgN = ?? this gets the righ value in set_PGN() below */
+	  /* (vm->state).pgN = ?? this gets the right value in set_PGN() below */
 	  (vm->state).cellN = (vm->state).rsm_cellN;
 	  link_values.command = PlayThis;
-	  link_values.data1 = (vm->state).rsm_blockN;
+	  link_values.data1 = (vm->state).rsm_blockN & 0xffff;
+	  link_values.data2 = (vm->state).rsm_blockN >> 16;
 	  if(!set_PGN(vm)) {
 	    /* Were at the end of the PGC, should not happen for a RSM */
 	    assert(0);
@@ -1403,9 +1421,10 @@ static int process_command(vm_t *vm, link_t link_values) {
       /* Stop SPRM9 Timer */
       /* Set SPRM1 and SPRM2 */
       assert((vm->state).domain == VMGM_DOMAIN || (vm->state).domain == FP_DOMAIN); /* ?? */
-      if(!set_TT(vm, link_values.data1))
-	assert(0);
-      link_values = play_PGC(vm);
+      if(set_TT(vm, link_values.data1))
+        link_values = play_PGC(vm);
+      else
+	link_values.command = Exit;
       break;
     case JumpVTS_TT:
       /* Jump to Title:data1 in same VTS Title Domain */
@@ -1462,7 +1481,8 @@ static int process_command(vm_t *vm, link_t link_values) {
 	  /* the normal case */
 	  assert((vm->state).domain == VMGM_DOMAIN || (vm->state).domain == FP_DOMAIN); /* ?? */
 	  (vm->state).domain = VTSM_DOMAIN;
-	  ifoOpenNewVTSI(vm, vm->dvd, link_values.data1);  /* Also sets (vm->state).vtsN */
+	  if (!ifoOpenNewVTSI(vm, vm->dvd, link_values.data1))  /* Also sets (vm->state).vtsN */
+	    assert(0);
 	} else {
 	  /* This happens on some discs like "Captain Scarlet & the Mysterons" or
 	   * the German RC2 of "Anatomie" in VTSM. */
@@ -1549,7 +1569,7 @@ static int process_command(vm_t *vm, link_t link_values) {
 #endif
     
   }
-  (vm->state).blockN = link_values.data1;
+  (vm->state).blockN = link_values.data1 | (link_values.data2 << 16);
   return 1;
 }
 
@@ -1575,8 +1595,9 @@ static int set_VTS_PTT(vm_t *vm, int vtsN, int vts_ttn, int part) {
   
   (vm->state).domain = VTS_DOMAIN;
 
-  if(vtsN != (vm->state).vtsN)
-    ifoOpenNewVTSI(vm, vm->dvd, vtsN);  /* Also sets (vm->state).vtsN */
+  if (vtsN != (vm->state).vtsN)
+    if (!ifoOpenNewVTSI(vm, vm->dvd, vtsN))  /* Also sets (vm->state).vtsN */
+      return 0;
   
   if ((vts_ttn < 1) || (vts_ttn > vm->vtsi->vts_ptt_srpt->nr_of_srpts) ||
       (part < 1) || (part > vm->vtsi->vts_ptt_srpt->title[vts_ttn - 1].nr_of_ptts) ) {
@@ -1601,6 +1622,9 @@ static int set_VTS_PTT(vm_t *vm, int vtsN, int vts_ttn, int part) {
 
 static int set_FP_PGC(vm_t *vm) {  
   (vm->state).domain = FP_DOMAIN;
+  if (!vm->vmgi->first_play_pgc) {
+    return set_PGCN(vm, 1);
+  }
   (vm->state).pgc = vm->vmgi->first_play_pgc;
   (vm->state).pgcN = vm->vmgi->vmgi_mat->first_play_pgc;
   return 1;
